tankers = {}

--------------------------------------------------------------------------
--------------------------------------------------------------------------
--Default tanker settings

tankers.AIRCRAFT_TYPE = "S-3B"
tankers.REFERENCE_OBJECT = "CVN-74"
tankers.COUNTRY = "USA"
tankers.ALTITUDE = 6000  -- feet
tankers.TRUE_AIRSPEED = 280  -- knots (250 kts IAS @ 6000 ft)
tankers.TRACK_START = 10  -- nautical miles
tankers.TRACK_LENGTH = 15  -- nautical miles
tankers.RADIO_FREQUENCY = 251
tankers.CALL_SIGN = "Arco"
tankers.TACAN_CHANNEL = "3Y"
tankers.TACAN_ID = "TKR"
tankers.LIVERY_ID = "default"
tankers.RESPAWN = true
tankers.USE_TACAN = true
tankers.USE_METRIC = false
tankers.INVISIBLE = false
tankers.NO_REACTION = false

--------------------------------------------------------------------------
--------------------------------------------------------------------------

tankers.active = {}
tankers.totalTankers = 0
tankers.checkInterval = 10

function tankers.getHeading(unit)
	local unitPosition = unit:getPosition()

	if unitPosition then
		local heading = math.atan2(unitPosition.x.z, unitPosition.x.x)
		if heading < 0 then
			heading = heading + 2 * math.pi
		end
		return heading
	end
end

function tankers.getTacanFreq(channel)

	local frequency = 0
	
	if channel < 64 then
		frequency = channel + 1087
	else
		frequency = channel + 961
	end

	return frequency * 1000000
end

function tankers.validateData(aircraftType, referenceObjectName, callsignName, countryID, tacanChannel, tacanUse)

	local validData = true

	if Unit.getByName(referenceObjectName) == nil then
		if StaticObject.getByName(referenceObjectName) == nil then
			trigger.action.outText("Tanker reference object does not exist:  " .. referenceObjectName  , 15)
			validData = false
		end
	end

	if aircraftType ~= "S-3B Tanker" and aircraftType ~= "KC-135" and aircraftType ~= "KC135MPRS" and aircraftType ~= "KC130" and aircraftType ~= "IL-78M" then
		trigger.action.outText("Invalid tanker aircraft type:  " .. aircraftType  , 15)
		trigger.action.outText("- Must be S-3B Tanker, KC-135, KC-135MPRS, KC-130, or IL-78M", 15)
		validData = false
	end

	local countryNumber = country.id[string.upper(countryID)]
	
	if countryNumber ~= nil then
		if coalition.getCountryCoalition(countryNumber) == coalition.side.BLUE then
			if callsignName ~= "Texaco" and callsignName ~= "Arco" and callsignName ~= "Shell" then
				trigger.action.outText("Invalid tanker callsign:  " .. callsignName  , 15)
				trigger.action.outText("- Must be Texaco, Arco, or Shell for Blue coalition", 15)
				validData = false
			end
		elseif coalition.getCountryCoalition(countryNumber) == coalition.side.RED then
			if tonumber(callsignName) == nil or tonumber(callsignName) < 1 or tonumber(callsignName) > 9 then
				trigger.action.outText("Invalid tanker callsign:  " .. callsignName  , 15)
				trigger.action.outText("- Must be a number between 0 and 9 for Red coalition", 15)
				validData = false
			end
		else
			trigger.action.outText("Invalid tanker country: " .. countryID, 10)
			validData = false
		end
	else
		trigger.action.outText("Invalid tanker country: " .. countryID, 10)
		validData = false
	end
	

	if tacanUse then

		local tacanChannelNumber = tonumber(string.sub(tacanChannel, 1, string.len(tacanChannel) - 1))
		local tacanChannelMode = string.upper(string.sub(tacanChannel, -1))

		if tacanChannelNumber == nil or tacanChannelNumber < 1 or tacanChannelNumber > 126 or (tacanChannelMode ~= "X" and tacanChannelMode ~= "Y") then
			trigger.action.outText("Invalid tanker TACAN channel:  " .. tacanChannel  , 15)
			validData = false
		end
	end
	
	-- if tacanChannel ~= false and tacanChannel ~= "false" then

		-- local tacanChan = tonumber(string.sub(tacanChannel, 1, string.len(tacanChannel) - 1))
		-- local tacanChanMode = string.upper(string.sub(tacanChannel, -1))

		-- if tacanChan == nil or tacanChan < 1 or tacanChan > 126 or (tacanChanMode ~= "X" and tacanChanMode ~= "Y") then
			-- trigger.action.outText("Invalid tanker TACAN channel:  " .. tacanChan  , 15)
			-- validData = false
		-- end
	-- end
	
	return validData

end

--function tankers.addTanker(aircraftType, referenceObjectName, countryID, altitude, trueAirspeed, trackStart, trackLength, radioFrequency, callsignName, tacanChannel, tacanID, liveryID, respawn)
function tankers.addTanker(properties)

if properties.AIRCRAFT_TYPE == nil then aircraftType = tankers.AIRCRAFT_TYPE else aircraftType = properties.AIRCRAFT_TYPE end
if properties.REFERENCE_OBJECT == nil then referenceObjectName = tankers.REFERENCE_OBJECT else referenceObjectName = properties.REFERENCE_OBJECT end
if properties.COUNTRY == nil then countryID = tankers.COUNTRY else countryID = properties.COUNTRY end
if properties.ALTITUDE == nil then altitude = tankers.ALTITUDE else altitude = properties.ALTITUDE end
if properties.TRUE_AIRSPEED == nil then trueAirspeed = tankers.TRUE_AIRSPEED else trueAirspeed = properties.TRUE_AIRSPEED end
if properties.TRACK_START == nil then trackStart = tankers.TRACK_START else trackStart = properties.TRACK_START end
if properties.TRACK_LENGTH == nil then trackLength = tankers.TRACK_LENGTH else trackLength = properties.TRACK_LENGTH end
if properties.RADIO_FREQUENCY == nil then radioFrequency = tankers.RADIO_FREQUENCY else radioFrequency = properties.RADIO_FREQUENCY end
if properties.CALL_SIGN == nil then callsignName = tankers.CALL_SIGN else callsignName = properties.CALL_SIGN end
if properties.TACAN_CHANNEL == nil then tacanChannel = tankers.TACAN_CHANNEL else tacanChannel = properties.TACAN_CHANNEL end
if properties.TACAN_ID == nil then tacanID = tankers.TACAN_ID else tacanID = properties.TACAN_ID end
if properties.LIVERY_ID == nil then liveryID = tankers.LIVERY_ID else liveryID = properties.LIVERY_ID end
if properties.RESPAWN == nil then respawn = tankers.RESPAWN else respawn = properties.RESPAWN end
if properties.USE_TACAN == nil then tacanUse = tankers.USE_TACAN else tacanUse = properties.USE_TACAN end
if properties.USE_METRIC == nil then metricUse = tankers.USE_METRIC else metricUse = properties.USE_METRIC end
if properties.INVISIBLE == nil then invisible = tankers.INVISIBLE else invisible = properties.INVISIBLE end
if properties.NO_REACTION == nil then noReaction = tankers.NO_REACTION else noReaction = properties.NO_REACTION end

	if not metricUse then
		altitude = altitude * 0.3048 -- convert feet to meters
		trueAirspeed = trueAirspeed * 0.514444 -- convert knots to meters per second
		trackStart = trackStart * 1852 -- convert nm to meters
		trackLength = trackLength * 1852 -- convert nm to meters
	end

	if aircraftType == "KC-130" then
		aircraftType = "KC130"
	elseif aircraftType == "S-3B" then
		aircraftType = "S-3B Tanker"
	elseif aircraftType == "IL-78" then
		aircraftType = "IL-78M"
	elseif aircraftType == "KC-135MPRS" then
		aircraftType = "KC135MPRS"
	end
		
	if tankers.validateData(aircraftType, referenceObjectName, callsignName, countryID, tacanChannel, tacanUse) then
		tankers.spawn(aircraftType, referenceObjectName, countryID, altitude, trueAirspeed, trackStart, trackLength, radioFrequency, callsignName, tacanChannel, tacanID, liveryID, respawn, tacanUse, metricUse, invisible, noReaction, nil, nil, nil)
	end

end

function tankers.spawn(aircraftType, referenceObjectName, countryID, altitude, trueAirspeed, trackStart, trackLength, radioFrequency, callsignName, tacanChannel, tacanID, liveryID, respawn, tacanUse, metricUse, invisible, noReaction, groupName, unitName, tableIndex)

	if aircraftType == "KC-130" then
		aircraftType = "KC130"
	elseif aircraftType == "S-3B" then
		aircraftType = "S-3B Tanker"
	elseif aircraftType == "IL-78" then
		aircraftType = "IL-78M"
	elseif aircraftType == "KC-135MPRS" then
		aircraftType = "KC135MPRS"
	end
		
	if tankers.totalTankers < 99 then
	
		local countryNumber = country.id[string.upper(countryID)]

		tankers.totalTankers = tankers.totalTankers + 1
		
		local referenceObject = Unit.getByName(referenceObjectName)
		if referenceObject == nil then
			referenceObject = StaticObject.getByName(referenceObjectName)
		end
		
		local referencePoint = referenceObject:getPoint()
		local referenceX = referencePoint.x
		local referenceY = referencePoint.z
		
		--local referenceHeading = (tankers.getHeading(referenceObject) + tankers.headingCorrection) % (2 * 3.14159265359)
		local referenceHeading = tankers.getHeading(referenceObject) % (2 * 3.14159265359)
		
		local referenceHeadingCos = math.cos(referenceHeading)
		local referenceHeadingSin = math.sin(referenceHeading)
		
		local spawnX = (referenceHeadingCos * (trackStart - 926)) + referenceX -- 926 meters = 0.5 nm
		local spawnY = (referenceHeadingSin * (trackStart - 926)) + referenceY -- 926 meters = 0.5 nm

		local trackStartX = (referenceHeadingCos * trackStart) + referenceX
		local trackStartY = (referenceHeadingSin * trackStart) + referenceY

		local trackEndX = (referenceHeadingCos * (trackStart + trackLength)) + referenceX
		local trackEndY = (referenceHeadingSin * (trackStart + trackLength)) + referenceY

		local fuel = 0
		local flares = 0
		local chaff = 0
		local totalTankers = 0
		
		local onboardNumber = string.format("%03d",tankers.totalTankers)

		if aircraftType == "S-3B Tanker" then
			fuel = "8000"
			--fuel = 5500
			flares = 30
			chaff = 30
		elseif aircraftType == "KC-135" or aircraftType == "KC135MPRS" then
			fuel = 90700
			flares = 0
			chaff = 0
		elseif aircraftType == "KC130" then
			fuel = 20830
			flares = 75
			chaff = 90
		elseif aircraftType == "IL-78M" then
			--fuel = "90000
			fuel = 90000
			flares = 96
			chaff = 96
		end
		
		-- local callsign1 = (tankers.totalTankers % 3) + 1

		local callsign = nil
		local callsignText = ""

		local callsign1 = 0
		local callsign2 = math.floor(tankers.totalTankers / 10)
		local callsign3 = tankers.totalTankers % 10

		
		if coalition.getCountryCoalition(countryNumber) == coalition.side.BLUE then
			if callsignName == "Texaco" then
				callsign1 = 1
			elseif callsignName == "Arco" then
				callsign1 = 2
			elseif callsignName == "Shell" then
				callsign1 = 3
			else
				callsign1 = tonumber(callsignName)
			end
			callsign =
			{
				[1] = callsign1,
				[2] = callsign2,
				[3] = callsign3,
				["name"] = callsignName .. tostring(callsign2) .. tostring(callsign3),
			}
			onboardNumber = tankers.totalTankers + (callsign1 * 100)
			callsignText = callsignName .. " " .. callsign2 .. callsign3
			else 
			callsign = tonumber(callsignName .. callsign2 .. callsign3)
			onboardNumber = callsign
			callsignText = callsign
		end
		
		local tankerGroupName = groupName
		local tankerUnitName = unitName

		if tankerGroupName == nil then
			tankerGroupName = "Tanker_Group_" .. tankers.totalTankers
		end

		if tankerUnitName == nil then
			tankerUnitName = "Tanker_Unit_" .. tankers.totalTankers
		end

		local tacanChannelNumber = 0
		local tacanChannelMode = ""
		local tacanCallsign = ""
		local tacanFrequency = 0
		local tacanSystem = 0

		if tacanUse then
			tacanChannelNumber = tonumber(string.sub(tacanChannel, 1, string.len(tacanChannel) - 1))
			tacanChannelMode = string.upper(string.sub(tacanChannel, -1))
			tacanCallsign = string.upper(string.sub(tacanChannel, 1, 3))
			tacanFrequency = tankers.getTacanFreq(tacanChannelNumber)
			if tacanChannelMode == "X" then
				tacanSystem = 4
			elseif tacanChannelMode == "Y" then
				tacanSystem = 5
			end
		end

		if noReaction then
			threatReaction = 0
		else
			threatReaction = 4
		end

		local groupData = 
		{
			["modulation"] = 0,
			["tasks"] = 
			{
			}, -- end of ["tasks"]
			["radioSet"] = false,
			["task"] = "Refueling",
			["uncontrolled"] = false,
			["route"] = 
			{
				["points"] = 
				{
					[1] = 
					{
						["alt"] = altitude,
						["action"] = "Turning Point",
						["alt_type"] = "BARO",
						["speed"] = trueAirspeed,
						["task"] = 
						{
							["id"] = "ComboTask",
							["params"] = 
							{
								["tasks"] = 
								{
									[1] = 
									{
										["number"] = 1,
										["auto"] = true,
										["id"] = "Tanker",
										["enabled"] = true,
										["params"] = 
										{
										}, -- end of ["params"]
									}, -- end of [1]
									[2] = 
									{
										["enabled"] = true,
										["auto"] = true,
										["id"] = "WrappedAction",
										["number"] = 2,
										["params"] = 
										{
											["action"] = 
											{
												["id"] = "ActivateBeacon",
												["params"] = 
												{
													["type"] = 4,
													["AA"] = true,
													["callsign"] = tacanCallsign,
													["system"] = tacanSystem,
													["channel"] = tacanChannelNumber,
													["modeChannel"] = tacanChannelMode,
													["bearing"] = true,
													["frequency"] = tacanFrequency
												}, -- end of ["params"]
											}, -- end of ["action"]
										}, -- end of ["params"]
									}, -- end of [2]
								}, -- end of ["tasks"]
							}, -- end of ["params"]
						}, -- end of ["task"]
						["type"] = "Turning Point",
						["y"] = spawnY,
						["x"] = spawnX,
						["name"] = "Tanker_Wpt_1_" .. tankers.totalTankers,
						["formation_template"] = "",
						["speed_locked"] = true,
					}, -- end of [1]
					[2] = 
					{
						["alt"] = altitude,
						["action"] = "Turning Point",
						["alt_type"] = "BARO",
						["speed"] = trueAirspeed,
						["task"] = 
						{
							["id"] = "ComboTask",
							["params"] = 
							{
								["tasks"] = 
								{
									[1] = 
									{
										["number"] = 1,
										["auto"] = false,
										["id"] = "Orbit",
										["enabled"] = true,
										["params"] = 
										{
											["altitude"] = altitude,
											["pattern"] = "Race-Track",
											["speed"] = trueAirspeed,
											["speedEdited"] = true,
										}, -- end of ["params"]
									}, -- end of [1]
								   [2] = 
									{
										["number"] = 2,
										["auto"] = false,
										["id"] = "WrappedAction",
										["enabled"] = true,
										["params"] = 
										{
											["action"] = 
											{
												["id"] = "SetInvisible",
												["params"] = 
												{
													["value"] = invisible,
												}, -- end of ["params"]
											}, -- end of ["action"]
										}, -- end of ["params"]
									}, -- end of [2]
									[3] = 
									{
										["number"] = 3,
										["auto"] = false,
										["id"] = "WrappedAction",
										["enabled"] = true,
										["params"] = 
										{
											["action"] = 
											{
												["id"] = "Option",
												["params"] = 
												{
													["value"] = threatReaction,
													["name"] = 1,
												}, -- end of ["params"]
											}, -- end of ["action"]
										}, -- end of ["params"]
									}, -- end of [3]
								}, -- end of ["tasks"]
							}, -- end of ["params"]
						}, -- end of ["task"]
						["type"] = "Turning Point",
						["y"] = trackStartY,
						["x"] = trackStartX,
						["name"] = "Tanker_Wpt_2_" .. tankers.totalTankers,
						["formation_template"] = "",
						["speed_locked"] = true,
					}, -- end of [2]
					[3] = 
					{
						["alt"] = altitude,
						["action"] = "Turning Point",
						["alt_type"] = "BARO",
						["speed"] = trueAirspeed,
						["task"] = 
						{
							["id"] = "ComboTask",
							["params"] = 
							{
								["tasks"] = 
								{
									[1] = 
									{
										["number"] = 1,
										["auto"] = false,
										["id"] = "WrappedAction",
										["enabled"] = true,
										["params"] = 
										{
											["action"] = 
											{
												["id"] = "Option",
												["params"] = 
												{
													["value"] = true,
													["name"] = 6,
												}, -- end of ["params"]
											}, -- end of ["action"]
										}, -- end of ["params"]
									}, -- end of [1]
								}, -- end of ["tasks"]
							}, -- end of ["params"]
						}, -- end of ["task"]
						["type"] = "Turning Point",
						["y"] = trackEndY,
						["x"] = trackEndX,
						["name"] = "Tanker_Wpt_3_" .. tankers.totalTankers,
						["formation_template"] = "",
						["speed_locked"] = true,
					}, -- end of [3]
					[4] = 
					{
						["alt"] = altitude / 2,
						["action"] = "Turning Point",
						["alt_type"] = "BARO",
						["speed"] = trueAirspeed,
						["task"] = 
						{
							["id"] = "ComboTask",
							["params"] = 
							{
								["tasks"] = 
								{
								}, -- end of ["tasks"]
							}, -- end of ["params"]
						}, -- end of ["task"]
						["type"] = "Turning Point",
						["y"] = referenceY,
						["x"] = referenceX,
						["name"] = "Tanker_Wpt_4_" .. tankers.totalTankers,
						["formation_template"] = "",
						["speed_locked"] = true,
					}, -- end of [4]
				}, -- end of ["points"]
			}, -- end of ["route"]
			--["groupId"] = 9999,
			["hidden"] = false,
			["units"] = 
			{
				[1] = 
				{
					["alt"] = altitude,
					["alt_type"] = "BARO",
					["livery_id"] = liveryID,
					["skill"] = "Excellent",
					["speed"] = trueAirspeed,
					["type"] = aircraftType,
					["psi"] = -referenceHeading,
					["y"] = spawnY,
					["x"] = spawnX,
					["name"] = tankerUnitName,
					["payload"] = 
					{
						["pylons"] = 
						{
						}, -- end of ["pylons"]
						["fuel"] = fuel,
						["flare"] = flares,
						["chaff"] = chaff,
						["gun"] = 100,
					}, -- end of ["payload"]
					["heading"] = referenceHeading,
					["callsign"] = callsign,
					["onboard_num"] = onboardNumber,
				}, -- end of [1]
			}, -- end of ["units"]
			["y"] = spawnY,
			["x"] = spawnX,
			["name"] = tankerGroupName,
			["communication"] = true,
			["start_time"] = 0,
			["frequency"] = radioFrequency,
		} -- end of groupData

		coalition.addGroup(countryNumber, Group.Category.AIRPLANE, groupData)

		local onStationAltitude = altitude
		if not metricUse then
			onStationAltitude = math.floor(onStationAltitude * 3.28084) -- convert meters to feet
		end
		trigger.action.outTextForCoalition(coalition.getCountryCoalition(countryNumber), "Tanker " .. callsignText .. " (" .. aircraftType .. ") on station at " .. onStationAltitude, 10)
		if tableIndex == nil then
			table.insert(tankers.active, {aircraftType, referenceObjectName, countryID, altitude, trueAirspeed, trackStart, trackLength, radioFrequency, callsignName, tacanChannel, tacanID, liveryID, respawn, tacanUse, metricUse, invisible, noReaction, groupData.name, groupData.units[1].name})
		else
			table.insert(tankers.active, tableIndex, {aircraftType, referenceObjectName, countryID, altitude, trueAirspeed, trackStart, trackLength, radioFrequency, callsignName, tacanChannel, tacanID, liveryID, respawn, tacanUse, metricUse, invisible, noReaction, groupData.name, groupData.units[1].name})
		end
	end

end
	
function tankers.isDead(groupName)

	if (Group.getByName(groupName) and Group.getByName(groupName):isExist() == false)
			or (Group.getByName(groupName) and #Group.getByName(groupName):getUnits() < 1)
			or not Group.getByName(groupName) then
		return true
	else
		return false
	end

end

function tankers.checkRespawn(arg, time)

	for i=1, #tankers.active do
		local altitude = tankers.active[i][4]
		local groupName = tankers.active[i][16]
		local unitName = tankers.active[i][17]
	
		--trigger.action.outText(callsignName .. " fuel - " .. Unit.getByName(unitName):getFuel(), 100)
		
		--if tankers.isDead(groupName) or Unit.getByName(unitName):getPosition().p.y < altitude - 600 then  -- 600 meters = 2000 feet
		if tankers.isDead(groupName) or Unit.getByName(unitName):getPosition().p.y < altitude - 275 then  -- 275 meters = 900 feet
			local aircraftType = tankers.active[i][1]
			local referenceObjectName = tankers.active[i][2]
			local countryID = tankers.active[i][3]
			--local altitude = tankers.active[i][4]
			local trueAirspeed = tankers.active[i][5]
			local trackStart = tankers.active[i][6]
			local trackLength = tankers.active[i][7]
			local radioFrequency = tankers.active[i][8]
			local callsignName = tankers.active[i][9]
			local tacanChannel = tankers.active[i][10]
			local tacanID = tankers.active[i][11]
			local liveryID = tankers.active[i][12]
			local respawn = tankers.active[i][13]
			local tacanUse = tankers.active[i][14]
			local metricUse = tankers.active[i][15]
			local invisible = tankers.active[i][16]
			local noReaction = tankers.active[i][17]
			--local groupName = tankers.active[i][16]
			--local unitName = tankers.active[i][17]

			table.remove(tankers.active, i)
			if respawn then
				if Unit.getByName(referenceObjectName) ~= nil then
					tankers.spawn(aircraftType, referenceObjectName, countryID, altitude, trueAirspeed, trackStart, trackLength, radioFrequency, callsignName, tacanChannel, tacanID, liveryID, respawn, tacanUse, metricUse, invisible, noReaction, groupName, unitName, i)				
				end
			end
		end
	end

	return time + tankers.checkInterval

end

do

	--tankers.addTanker("IL-78M", "Tire1", "Russia", 20000, 300, 20, 30, 270, 5, "10X", true)
	--tankers.addTanker("KC-130", "CVN-70", "USA", 15000, 300, 1, 20, 251, "Texaco", "12X", true)

	timer.scheduleFunction(tankers.checkRespawn, nil, timer.getTime() + tankers.checkInterval)

end